@@ -85,6 +85,10 @@ void Battle::run(NewNationPara *mpGame, int mpPlayerCount)
 	}
 #endif
 
+#ifdef HEADLESS_SIM
+	game.game_mode = GAME_DEMO; // skip end screens
+#endif
+
 	// ####### begin Gilbert 24/10 #######//
 	//-- random seed is initalized at connecting multiplayer --//
 	//if( !mpGame )
@@ -106,12 +110,14 @@ void Battle::run(NewNationPara *mpGame, int mpPlayerCount)
 
 	if( !mpGame )
 	{
+#ifndef HEADLESS_SIM
 		// if config.race_id == 0, select a random race, but don't call misc.random
 		int nationRecno = nation_array.new_nation( NATION_OWN,
 								config.race_id ? config.race_id : 1+misc.get_time() % MAX_RACE,
 								config.player_nation_color );
 
 		nation_array.set_human_name( nationRecno, config.player_name );
+#endif
 	}
 	else
 	{
@@ -136,7 +142,11 @@ void Battle::run(NewNationPara *mpGame, int mpPlayerCount)
 	}
 	else
 	{
+#ifdef HEADLESS_SIM
+		create_ai_nation(config.ai_nation_count+1); // no human player
+#else
 		create_ai_nation(config.ai_nation_count);
+#endif
 	}
 
 	//------ create pregame objects ------//
@@ -167,8 +177,10 @@ void Battle::run(NewNationPara *mpGame, int mpPlayerCount)
 	// Set speed to normal. When hosting, broadcast the speed to the clients. As a client, set to default speed initially.
 	if ( remote.is_enable() && !remote.is_host )
 		sys.set_speed(12, COMMAND_REMOTE);
+#ifndef HEADLESS_SIM
 	else
 		sys.set_speed(12, COMMAND_PLAYER);
+#endif
 
 	//---- reset cheats ----//
 
@@ -196,8 +208,10 @@ void Battle::run(NewNationPara *mpGame, int mpPlayerCount)
 
 	//------- enable/disable sound effects -------//
 
+#ifndef HEADLESS_SIM
 	int songId = (~nation_array)->race_id <= 7 ? (~nation_array)->race_id+1 : music.random_bgm_track();
 	music.play(songId, sys.cdrom_drive ? MUSIC_CD_THEN_WAV : 0 );
+#endif
 
 	mouse_cursor.restore_icon(oldCursor);
 
