@@ -160,6 +160,7 @@ Sys::Sys()
    is_mp_game = 0;
    toggle_full_screen_flag = 0;
    user_pause_flag = 0;
+   disp_fps_flag = 0;
 }
 //----------- End of function Sys::Sys -----------//
 
@@ -256,7 +257,7 @@ int Sys::init_directx()
       return FALSE;
    DEBUG_LOG("vga.init() ok");
 #if !defined(DEBUG) && !defined(_DEBUG)
-   vga.toggle_full_screen();
+   vga.set_full_screen_mode(1);
 #endif
 
    //---------- Initialize Audio ----------//
@@ -434,7 +435,7 @@ int Sys::set_config_dir()
    // create the config directory
    if (!misc.mkpath(dir_config))
    {
-      ERR("Unable to acquire a usable game config dir.\n");
+      show_error_dialog(_("Unable to determine a location for storing the game configuration."));
       dir_config[0] = 0;
       return 0;
    }
@@ -683,6 +684,9 @@ void Sys::main_loop(int isLoadedGame)
          vga_front.lock_buf();
 
          yield();       // could be improved, give back the control to Windows, so it can do some OS management. Maybe call WaitMessage() here and set up a timer to get messages regularly.
+#ifndef HEADLESS_SIM
+         vga.flip();
+#endif
 
          detect();
 
@@ -774,7 +778,9 @@ void Sys::main_loop(int isLoadedGame)
                // second condition (markTime-lastDispFrameTime >= DWORD(1000/config.frame_speed) )
                // may happen in multiplayer, where 'should_next_frame' would pass (what means it's time
                // to process new frame according to config.frame_speed), but 'is_mp_sync' still failed.
+#ifndef HEADLESS_SIM
                disp_frame();
+#endif
                lastDispFrameTime = markTime;
 
 					// ####### patch begin Gilbert 17/11 ######//
@@ -1010,6 +1016,9 @@ void Sys::auto_save()
 //
 void Sys::pause()
 {
+#ifdef HEADLESS_SIM
+   return;
+#endif
    if( config.frame_speed && sys_flag == SYS_RUN )
    {
       set_speed( 0 );
@@ -1034,6 +1043,28 @@ void Sys::unpause()
 //--------- End of function Sys::unpause ---------//
 
 
+//-------- Begin of function Sys::show_error_dialog ----------//
+//
+// Show SDL error dialog that does not depend on video init. This is a blocking
+// routine, and never should be used after init.
+//
+void Sys::show_error_dialog(const char *formatStr, ...)
+{
+   enum { RESULT_STR_LEN=200 };
+
+   static char resultStr[RESULT_STR_LEN+1];
+
+   va_list argPtr;
+
+   va_start( argPtr, formatStr );
+   vsnprintf( resultStr, RESULT_STR_LEN, formatStr, argPtr );
+   va_end( argPtr );
+
+   SDL_ShowSimpleMessageBox( SDL_MESSAGEBOX_ERROR, "Seven Kingdoms", resultStr, NULL );
+}
+//----------- End of function Sys::show_error_dialog ----------//
+
+
 //-------- Begin of function Sys::yield --------//
 //
 void Sys::yield()
@@ -1050,7 +1081,7 @@ void Sys::yield()
    if (toggle_full_screen_flag)
    {
       toggle_full_screen_flag = 0;
-      vga.toggle_full_screen();
+      vga.set_full_screen_mode(-1);
    }
 
    mouse.poll_event();
@@ -1092,8 +1123,6 @@ void Sys::yield()
       }
    }
 
-   vga.flip();
-
    isYielding=0;
 }
 //--------- End of function Sys::yield ---------//
@@ -2673,7 +2702,7 @@ int Sys::chdir_to_game_dir()
       return 1;
 #endif
 
-   ERR("Unable to locate the game data.\n");
+   show_error_dialog(_("Unable to locate the game resources."));
 
    return 0;
 }
